/* automaton.c */

#include <glib.h>
#include <gdk/gdk.h>
#include <gtk/gtkmain.h>

#include "automaton.h"

Automaton *automaton_create(gint num)
{
	Automaton *atm;

	g_return_val_if_fail(num > 0, NULL);

	atm = g_new0(Automaton, 1);
	atm->max = num - 1;
	atm->state = g_new0(AtmState, num);

	return atm;
}

void automaton_destroy(Automaton *atm)
{
	if (!atm) return;

	g_free(atm->state);
	g_free(atm);
}

void automaton_input_cb(gpointer data, gint source,
			GdkInputCondition condition)
{
	Automaton *atm = (Automaton *)data;
	gint next;

	//g_print("atm->num = %d\n", atm->num);

	if (atm->timeout_tag > 0) {
		gtk_timeout_remove(atm->timeout_tag);
		atm->timeout_tag = 0;
		atm->elapsed = 0;
	}
	gdk_input_remove(atm->tag);
	atm->tag = 0;

	next = atm->state[atm->num].handler(source, atm->data);
	if (next >= 0 && next <= atm->max && next != atm->num) {
		atm->num = next;
		atm->tag = gdk_input_add(source,
					 atm->state[atm->num].condition,
					 automaton_input_cb,
					 data);
	} else {
		atm->terminate(source, data);
	}
}
