/* filesel.c */

#include <glib.h>
#include <gdk/gdkkeysyms.h>
#include <gtk/gtkwidget.h>
#include <gtk/gtkfilesel.h>
#include <gtk/gtkmain.h>
#include <gtk/gtksignal.h>

#include "filesel.h"
#include "manage_window.h"
#include "gtkutils.h"

static GtkWidget *filesel;
static gboolean filesel_ack;

static void filesel_create(const gchar *title);
static void filesel_ok_cb(GtkWidget *widget, gpointer data);
static void filesel_cancel_cb(GtkWidget *widget, gpointer data);
static void key_pressed(GtkWidget *widget, GdkEventKey *event, gpointer data);

gchar *filesel_select_file(const gchar *title, const gchar *file)
{
	static gchar *filename = NULL;

	filesel_create(title);

	manage_window_set_transient(GTK_WINDOW(filesel));

	if (!filename) {
		gchar *home;

		home = g_strconcat(g_get_home_dir(), G_DIR_SEPARATOR_S, NULL);
		gtk_file_selection_set_filename(GTK_FILE_SELECTION(filesel),
						home);
		g_free(home);
	} else {
		g_free(filename);
		filename = NULL;
	}

	if (file)
		gtk_file_selection_set_filename(GTK_FILE_SELECTION(filesel),
						file);

	gtk_widget_show(filesel);

	gtk_main();

	if (filesel_ack) {
		gchar *str;

		str = gtk_file_selection_get_filename
			(GTK_FILE_SELECTION(filesel));
		if (str && str[0] != '\0')
			filename = g_strdup(str);
	}

	gtk_widget_destroy(filesel);
	GTK_EVENTS_FLUSH();

	return filename;
}

static void filesel_create(const gchar *title)
{
	filesel = gtk_file_selection_new(title);
	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(filesel)->ok_button),
			   "clicked", GTK_SIGNAL_FUNC(filesel_ok_cb),
			   NULL);
	gtk_signal_connect
		(GTK_OBJECT(GTK_FILE_SELECTION(filesel)->cancel_button),
		 "clicked", GTK_SIGNAL_FUNC(filesel_cancel_cb),
		 NULL);
	gtk_signal_connect(GTK_OBJECT(filesel), "delete_event",
			   GTK_SIGNAL_FUNC(filesel_cancel_cb),
			   NULL);
	gtk_signal_connect(GTK_OBJECT(filesel), "key_press_event",
			   GTK_SIGNAL_FUNC(key_pressed), NULL);
	gtk_signal_connect(GTK_OBJECT(filesel), "focus_in_event",
			   GTK_SIGNAL_FUNC(manage_window_focus_in), NULL);
	gtk_signal_connect(GTK_OBJECT(filesel), "focus_out_event",
			   GTK_SIGNAL_FUNC(manage_window_focus_out), NULL);

	gtk_window_set_modal(GTK_WINDOW(filesel), TRUE);
}

static void filesel_ok_cb(GtkWidget *widget, gpointer data)
{
	filesel_ack = TRUE;
	gtk_main_quit();
}

static void filesel_cancel_cb(GtkWidget *widget, gpointer data)
{
	filesel_ack = FALSE;
	gtk_main_quit();
}

static void key_pressed(GtkWidget *widget, GdkEventKey *event, gpointer data)
{
	if (event && event->keyval == GDK_Escape)
		filesel_cancel_cb(NULL, NULL);
}
