/* main.c */

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include <glib.h>
#include <gtk/gtkmain.h>
#include <gtk/gtkrc.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <time.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <signal.h>
#if HAVE_LOCALE_H
#  include <locale.h>
#endif

#include "intl.h"
#include "main.h"
#include "mainwindow.h"
#include "folderview.h"
#include "summaryview.h"
#include "prefs_common.h"
#include "prefs_account.h"
#include "account.h"
#include "procmsg.h"
#include "inc.h"
#include "import.h"
#include "alertpanel.h"
#include "addressbook.h"
#include "utils.h"

gchar *prog_version;
gchar *maildir;
gboolean debug_mode = FALSE;

#if 0
_("can't make %s folder.\n")
#endif

#define MAKE_DIR_IF_NOT_EXIST(dir) \
{ \
	if (!is_dir_exist(dir)) { \
		if (mkdir(dir, S_IRWXU) < 0) { \
			perror("mkdir"); \
			g_warning(_("can't make %s folder.\n"), dir); \
			return 1; \
		} \
		if (chmod(dir, S_IRWXU) < 0) \
			FILE_OP_ERROR(dir, "chmod"); \
	} \
}

int main(int argc, char *argv[])
{
	gchar *userrc;
	MainWindow *mainwin;
	FolderView *folderview;

	//if (!g_thread_supported()) g_thread_init(NULL);

	setlocale(LC_ALL, "");
	bindtextdomain(PACKAGE, LOCALEDIR);
	textdomain(PACKAGE);

	gtk_set_locale();
	gtk_init(&argc, &argv);

	/* parse gtkrc files */
	gtk_rc_parse("/etc/gtkrc");
	userrc = g_strconcat(g_get_home_dir(), G_DIR_SEPARATOR_S, ".gtkrc",
			     NULL);
	gtk_rc_parse(userrc);
	g_free(userrc);
	userrc = g_strconcat(g_get_home_dir(), G_DIR_SEPARATOR_S, ".gtk",
			     G_DIR_SEPARATOR_S, "gtkrc", NULL);
	gtk_rc_parse(userrc);
	g_free(userrc);
	userrc = g_strconcat(g_get_home_dir(), G_DIR_SEPARATOR_S, RC_DIR,
			     G_DIR_SEPARATOR_S, "gtkrc", NULL);
	gtk_rc_parse(userrc);
	g_free(userrc);

	gtk_rc_parse("./gtkrc");

	if (argc > 1) {
		if (!strncmp(argv[1], "--debug", 7))
			debug_mode = TRUE;
	}

	prog_version = PROG_VERSION;
	maildir = g_strconcat(g_get_home_dir(), G_DIR_SEPARATOR_S, MAIL_DIR,
			      NULL);

	CHDIR_RETURN_VAL_IF_FAIL(g_get_home_dir(), 1);

	/* backup if old rc file exists */
	if (is_file_exist(RC_DIR)) {
		if (rename(RC_DIR, RC_DIR".bak") < 0)
			FILE_OP_ERROR(RC_DIR, "rename");
	}
	MAKE_DIR_IF_NOT_EXIST(RC_DIR);
	MAKE_DIR_IF_NOT_EXIST(get_news_cache_dir());

	MAKE_DIR_IF_NOT_EXIST(MAIL_DIR);
	CHDIR_RETURN_VAL_IF_FAIL(maildir, 1);
	MAKE_DIR_IF_NOT_EXIST(INBOX_DIR);
	MAKE_DIR_IF_NOT_EXIST(OUTBOX_DIR);
	MAKE_DIR_IF_NOT_EXIST(QUEUE_DIR);
	MAKE_DIR_IF_NOT_EXIST(DRAFT_DIR);
	MAKE_DIR_IF_NOT_EXIST(TRASH_DIR);

	srandom((gint)time(NULL));

	prefs_common_read_config();
	prefs_common_save_config();

	mainwin = main_window_create
		(prefs_common.sep_folder | prefs_common.sep_msg << 1);
	folderview = mainwin->folderview;

	account_read_config_all();
	account_save_config_all();

	folderview_set(folderview);

	if (prefs_common.chk_on_startup) {
		inc_mail(mainwin);
	} else {
		gtk_ctree_select(GTK_CTREE(folderview->ctree),
				 folderview->mailbox);
		gtk_widget_grab_focus(folderview->ctree);
	}

	/* ignore SIGPIPE signal for preventing sudden death of program */
	signal(SIGPIPE, SIG_IGN);

	gtk_main();

	return 0;
}

void app_will_exit(GtkWidget *widget, gpointer data)
{
	MainWindow *mainwin = data;

	if (prefs_common.clean_on_exit)
		main_window_clean_trash(mainwin);

	/* save all state before exiting */
	folderview_write_cache(mainwin->folderview);
	summary_write_cache(mainwin->summaryview);

	main_window_get_size(mainwin);
	main_window_get_position(mainwin);
	prefs_common_save_config();
	account_save_config_all();
	addressbook_export_to_file();
}
