#ifndef __PROCMIME_H__
#define __PROCMIME_H__

#include <glib.h>

typedef struct _MimeInfo	MimeInfo;

typedef enum
{
	ENC_7BIT,
	ENC_QUOTED_PRINTABLE,
	ENC_BASE64,
	ENC_UNKNOWN
} EncodingType;

typedef enum
{
	MIME_TEXT,
	MIME_MESSAGE_RFC822,
	MIME_APPLICATION,
	MIME_MULTIPART,
	MIME_UNKNOWN
} MimeType;

struct _MimeInfo
{
	gchar *encoding;

	EncodingType encoding_type;
	MimeType mime_type;

	gchar *content_type;
	gchar *charset;
	gchar *name;
	gchar *boundary;

	gchar *content_disposition;
	gchar *filename;

	glong fpos;
	guint size;

	MimeInfo *sub;
	MimeInfo *parent;
};

#define IS_BOUNDARY(s, bnd, len) \
	(s[0] == '-' && s[1] == '-' && !strncmp(s + 2, bnd, len))

MimeInfo *procmime_mimeinfo_new		(void);
void procmime_mimeinfo_free		(MimeInfo	*mimeinfo);
void procmime_scan_encoding		(MimeInfo	*mimeinfo,
					 const gchar	*encoding);
void procmime_scan_content_type		(MimeInfo	*mimeinfo,
					 const gchar	*content_type);
void procmime_scan_content_disposition	(MimeInfo	*mimeinfo,
					 const gchar	*content_disposition);
MimeInfo *procmime_scan_mime_header	(FILE		*fp);
FILE *procmime_decode_content		(FILE		*outfp,
					 FILE		*infp,
					 MimeInfo	*mimeinfo);

#endif /* __PROCMIME_H__ */
